/*
 * Copyright (c) 2005-2007 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package fr.obeo.acceleo.chain.impl.spec;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.ecore.EObject;

import fr.obeo.acceleo.chain.ModelSet;
import fr.obeo.acceleo.chain.impl.GenerateImpl;
import fr.obeo.acceleo.chain.tools.CObject;
import fr.obeo.acceleo.chain.tools.ChainLog;
import fr.obeo.acceleo.ecore.factories.FactoryException;
import fr.obeo.acceleo.ecore.tools.ELoaderUtils;
import fr.obeo.acceleo.ecore.tools.ETools;
import fr.obeo.acceleo.gen.IGenFilter;
import fr.obeo.acceleo.gen.phantom.PhantomResources;
import fr.obeo.acceleo.gen.phantom.SyncPhantom;
import fr.obeo.acceleo.gen.template.Template;
import fr.obeo.acceleo.gen.template.TemplateSyntaxExceptions;
import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeException;
import fr.obeo.acceleo.gen.template.eval.LaunchManager;
import fr.obeo.acceleo.gen.template.eval.log.Trace;
import fr.obeo.acceleo.gen.template.eval.merge.MergeTools;
import fr.obeo.acceleo.gen.template.scripts.AbstractScript;
import fr.obeo.acceleo.gen.template.scripts.EmptyScript;
import fr.obeo.acceleo.gen.template.scripts.IScript;
import fr.obeo.acceleo.gen.template.scripts.SpecificScript;
import fr.obeo.acceleo.tools.classloaders.AcceleoClassLoader;
import fr.obeo.acceleo.tools.plugins.AcceleoModuleProvider;
import fr.obeo.acceleo.tools.resources.FileContentMap;
import fr.obeo.acceleo.tools.resources.Resources;

/**
 * A representation of the model object '<em><b>Generate</b></em>'. It
 * generates files for a model and a script.
 * 
 * @author www.obeo.fr
 * 
 */
public class CGenerate extends GenerateImpl implements CObject {

	/**
	 * Preview backup folder.
	 */
	public static final String PHANTOM_PREVIEW_FOLDER_BACKUP = PhantomResources.getRootName() + "/preview/backup";

	/**
	 * Preview generated folder.
	 */
	public static final String PHANTOM_PREVIEW_FOLDER_GENERATED = PhantomResources.getRootName() + "/preview/generated";

	/**
	 * Clears the context.
	 */
	public static void clearModelContext() {
		modelContext.clear();
	}

	private static FileContentMap modelContext = new FileContentMap();

	/**
	 * Cache size to get models more quickly.
	 */
	private final static int MODEL_CACHE_SIZE = 3;

	private static FileContentMap nsUriModelContext = new FileContentMap(MODEL_CACHE_SIZE);

	/* (non-Javadoc) */
	public void launch(CChain cchain, IGenFilter genFilter, IProgressMonitor monitor, LaunchManager mode) throws CoreException {
		StringBuffer report = new StringBuffer("");
		IContainer container = ResourcesPlugin.getWorkspace().getRoot();
		try {
			if (!ETools.validateURI(getMetamodel().getPath(), false)) {
				report.append("Metamodel " + getMetamodel().getPath() + " doesn't exist\n");
			} else {
				List modelList = new ArrayList();
				if (getModel() instanceof ModelSet) {
					IContainer modelSet = Resources.getContainer(container, new Path(getModel().getPath()), monitor);
					if (modelSet != null && modelSet.exists()) {
						IFile[] files = Resources.members(modelSet);
						for (int i = 0; i < files.length; i++) {
							IFile file = files[i];
							modelList.add(file.getFullPath().toString());
						}
					} else {
						report.append("The folder " + getModel().getPath() + " doesn't exist\n");
					}
				} else if (getModel() != null) {
					modelList.add(getModel().getPath());
				}
				Iterator modelIt = modelList.iterator();
				while (modelIt.hasNext()) {
					String modelPath = (String) modelIt.next();
					IFile model = container.getFile(new Path(modelPath));
					if (!model.exists()) {
						report.append("Model " + modelPath + " doesn't exist\n");
					} else {
						monitor.subTask("Open model " + modelPath);
						EObject object;
						if (ETools.validateURI(getMetamodel().getPath(), true)) {
							object = (EObject) nsUriModelContext.get(model);
							if (object == null) {
								object = ETools.loadXMI(modelPath);
								nsUriModelContext.put(model, object);
							}
						} else {
							object = (EObject) modelContext.get(model);
							if (object == null) {
								ELoaderUtils.initModelFactories(model, CGenerate.class.getClassLoader());
								object = ETools.loadXMI(modelPath);
								modelContext.put(model, object);
							}
						}
						AcceleoClassLoader.setPreferredLoader(object);
						try {
							AbstractScript defaultGenSettings = createDefaultScript();
							IScript aScript = defaultGenSettings;
							if (getGenerator() != null && getGenerator().getPath() != null && getGenerator().getPath().length() > 0) {
								File script = AcceleoModuleProvider.getDefault().getFile(new Path(getGenerator().getPath()));
								if (script == null || !script.exists()) {
									report.append("Template " + getGenerator().getPath() + " doesn't exist\n");
								} else {
									aScript = new SpecificScript(script, cchain.getFile().getLocation().toFile());
									aScript.reset();
									aScript.addImport(defaultGenSettings);
									defaultGenSettings.setSpecific(aScript);
								}
							}
							if (aScript.isDefault() || aScript.isSpecific()) {
								IContainer targetBackup;
								IContainer targetGenerated;
								IContainer target;
								if (mode.getMode() == LaunchManager.PREVIEW_MODE) {
									IFile chainFile = cchain.getFile();
									IProject project = (chainFile != null) ? chainFile.getProject() : container.getProject();
									targetBackup = Resources.getContainerOrCreateFolder(project, new Path(PHANTOM_PREVIEW_FOLDER_BACKUP).append(getFolder().getPath()), monitor);
									targetGenerated = Resources.getContainerOrCreateFolder(project, new Path(PHANTOM_PREVIEW_FOLDER_GENERATED).append(getFolder().getPath()), monitor);
									target = Resources.getContainer(container, new Path(getFolder().getPath()), monitor);
								} else {
									targetBackup = null;
									targetGenerated = null;
									if (mode.getMode() == LaunchManager.PHANTOM_MODE) {
										target = Resources.getContainer(container, new Path(getFolder().getPath()), monitor);
									} else {
										target = Resources.getContainerOrCreateFolder(container, new Path(getFolder().getPath()), monitor);
									}
								}
								List files = generate(modelPath, cchain, genFilter, aScript, target, targetBackup, targetGenerated, object, true, report, monitor, mode);
								cchain.addGeneratedFiles(files);
							} else {
								report.append("Generative template not found\n");
							}
						} finally {
							AcceleoClassLoader.setPreferredLoader(null);
						}
					}
				}
			}
		} catch (TemplateSyntaxExceptions e) {
			report.append(e.getMessage());
		} catch (FactoryException e) {
			report.append(e.getMessage());
		} catch (ENodeException e) {
			report.append(e.getMessage());
		}
		ChainLog.report(cchain, this, report, monitor, mode);
		monitor.worked(10);
	}

	private AbstractScript createDefaultScript() {
		return new EmptyScript();
	}

	private List generate(String modelPath, CChain cchain, IGenFilter genFilter, IScript aScript, IContainer target, IContainer targetBackup, IContainer targetGenerated, EObject object,
			boolean recursive, StringBuffer report, IProgressMonitor monitor, LaunchManager mode) throws FactoryException, ENodeException, CoreException {
		if (monitor.isCanceled()) {
			throw new OperationCanceledException();
		}
		List result = new ArrayList();
		if (aScript.isGenerated(object)) {
			IPath path = aScript.getFilePath(object, true);
			// path != null => Generate file
			if (path != null) {
				if (genFilter.filter(aScript.getFile(), target.getFile(path), object)) {
					monitor.subTask("Try to generate file : " + target.getFile(path).getFullPath().toString());
					if (mode.getMode() == LaunchManager.DEBUG_MODE) {
						System.out.println("Try to generate file : " + target.getFile(path).getFullPath().toString());
					}
					ENode eval = evaluate(aScript, object, mode);
					if (eval.log().hasError()) {
						report.append("-> Errors in file : ");
						report.append(path);
						report.append("\n");
						report.append(eval.log().toString());
						report.append("\n");
					}
					StringBuffer buffer = new StringBuffer(eval.asString());
					StringBuffer oldBuffer = Resources.getFileContent(target.getFile(path), false);
					if (oldBuffer.length() > 0) {
						String lostCode = MergeTools.merge(target.getFile(path), buffer, oldBuffer, MergeTools.DEFAULT_USER_BEGIN, MergeTools.DEFAULT_USER_END);
						if (lostCode.length() > 0) {
							lostCode = "[Lost Code] " + new Date().toString() + "\n" + lostCode + "\n";
							if (mode.getMode() == LaunchManager.PREVIEW_MODE) {
								Resources.appendFile(targetGenerated, path.addFileExtension(MergeTools.LOST_FILE_EXTENSION), lostCode, monitor).setDerived(true);
							} else if (mode.getMode() != LaunchManager.PHANTOM_MODE) {
								Resources.appendFile(target, path.addFileExtension(MergeTools.LOST_FILE_EXTENSION), lostCode, monitor).setDerived(true);
							}
						}
					}
					if (mode.getMode() == LaunchManager.PREVIEW_MODE) {
						if (targetBackup != null) {
							Resources.createFile(targetBackup, path, oldBuffer.toString(), monitor);
						}
						IFile generated = Resources.createFile(targetGenerated, path, buffer.toString(), monitor);
						result.add(generated);
					} else {
						IFile generated;
						if (mode.getMode() == LaunchManager.PHANTOM_MODE) {
							generated = Resources.getFile(target, path, monitor);
						} else {
							generated = Resources.createFile(target, path, buffer.toString(), monitor);
						}
						SyncPhantom.createPhantomExtensionPoint(generated, ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(modelPath)), aScript.getFile(), target, cchain.getFile(), eval,
								monitor);
						result.add(generated);
					}
				} else if (target.getFile(path).exists()) {
					List files = new ArrayList();
					files.add(target.getFile(path));
					cchain.addGeneratedFiles(files);
				}
			}
		}
		if (recursive)
			result.addAll(generateSub(modelPath, cchain, genFilter, aScript, target, targetBackup, targetGenerated, object, report, monitor, mode));
		return result;
	}

	private List generateSub(String modelPath, CChain cchain, IGenFilter genFilter, IScript aScript, IContainer target, IContainer targetBackup, IContainer targetGenerated, EObject object,
			StringBuffer report, IProgressMonitor monitor, LaunchManager mode) throws FactoryException, ENodeException, CoreException {
		List result = new ArrayList();
		Iterator contents = object.eContents().iterator();
		while (contents.hasNext()) {
			EObject content = (EObject) contents.next();
			result.addAll(generate(modelPath, cchain, genFilter, aScript, target, targetBackup, targetGenerated, content, true, report, monitor, mode));
		}
		return result;
	}

	private ENode evaluate(IScript aScript, EObject object, LaunchManager mode) throws FactoryException, ENodeException {
		Trace.TRACE = mode.getMode() == LaunchManager.DEBUG_MODE;
		Template template = aScript.getRootTemplate(object, true);
		if (template != null) {
			boolean withComment = aScript.isDefault() || !aScript.hasFileTemplate();
			if (withComment) {
				return template.evaluateWithComment(object, mode);
			} else {
				return template.evaluate(object, mode);
			}
		} else {
			return new ENode(ENode.EMPTY, object, Template.EMPTY, mode.isSynchronize());
		}
	}

	/* (non-Javadoc) */
	public int totalWork() {
		return 10;
	}

}
