/*
 * Copyright (c) 2005-2007 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package fr.obeo.acceleo.chain.runner;

import java.io.File;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPlatformRunnable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.ecore.EObject;

import fr.obeo.acceleo.chain.impl.spec.CChain;
import fr.obeo.acceleo.chain.tools.CLoaderUtils;
import fr.obeo.acceleo.gen.AcceleoEcoreGenPlugin;
import fr.obeo.acceleo.gen.IGenFilter;
import fr.obeo.acceleo.gen.template.eval.LaunchManager;
import fr.obeo.acceleo.tools.resources.Resources;

/**
 * Platform runnables represent executable entry points into plug-ins. This
 * class can be used to launch a chain file outside of eclipse.
 * <p>
 * The command to launch all chain files : java -cp startup.jar
 * org.eclipse.core.launcher.Main -application fr.obeo.acceleo.chain.launch
 * -data [workspace-directory]
 * <p>
 * The command to launch one chain file : java -cp startup.jar
 * org.eclipse.core.launcher.Main -application fr.obeo.acceleo.chain.launch
 * -data [workspace-directory] -chain [chain-file]
 * <p>
 * 
 * @author www.obeo.fr
 * 
 */
public class ChainRunner implements IPlatformRunnable {

	/* (non-Javadoc) */
	public Object run(Object args) throws Exception {
		try {
			final String[] arguments = (String[]) args;
			final IWorkspace workspace = ResourcesPlugin.getWorkspace();
			IWorkspaceRunnable runnable = new IWorkspaceRunnable() {
				public void run(IProgressMonitor monitor) throws CoreException {
					try {
						String chainFilename = null;
						for (int index = 0; index < arguments.length && arguments[index].startsWith("-"); ++index) {
							if (arguments[index].equalsIgnoreCase("-chain")) {
								chainFilename = arguments[++index];
							}
						}
						monitor.beginTask("Refresh workspace", 1);
						workspace.getRoot().refreshLocal(IResource.DEPTH_INFINITE, monitor);
						if (chainFilename != null) {
							monitor.beginTask("Launch chain file : " + chainFilename, 1);
							System.out.println("Launch chain file : " + chainFilename);
							IFile chainFile = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(chainFilename));
							launch(monitor, chainFile);
						} else {
							IProject[] projects = workspace.getRoot().getProjects();
							for (int i = 0; i < projects.length; i++) {
								IProject project = projects[i];
								if (project.isAccessible()) {
									IFile[] chainFiles = Resources.members(project, new String[] { "chain" });
									monitor.beginTask("Launch all the chain files of the project '" + project.getName() + "'", chainFiles.length);
									System.out.println("Launch all the chain files of the project '" + project.getName() + "'");
									for (int j = 0; j < chainFiles.length; j++) {
										launch(monitor, chainFiles[j]);
									}
								}
							}
						}
					} finally {
						monitor.done();
					}
				}

				private void launch(IProgressMonitor monitor, IFile chainFile) {
					CChain chain = (chainFile.isAccessible()) ? CLoaderUtils.getCChainForChainFile(chainFile) : null;
					if (chain != null) {
						try {
							IGenFilter genFilter = new IGenFilter() {
								public boolean filter(File script, IFile targetFile, EObject object) throws CoreException {
									return true;
								}
							};
							chain.launch(genFilter, monitor, LaunchManager.create("run", true));
						} catch (CoreException e) {
							System.out.println("'" + chainFile.getFullPath().toString() + "' throws :");
							e.printStackTrace();
							AcceleoEcoreGenPlugin.getDefault().log(e, true);
						}
					} else {
						System.out.println("Specified chain does not exist : " + chainFile.getFullPath().toString());
						AcceleoEcoreGenPlugin.getDefault().log("Specified chain does not exist : " + chainFile.getFullPath().toString(), true);
					}
					monitor.worked(1);
				}
			};
			workspace.run(runnable, new NullProgressMonitor());
			return IPlatformRunnable.EXIT_OK;
		} catch (CoreException e) {
			printUsage();
			e.printStackTrace();
			AcceleoEcoreGenPlugin.getDefault().log(e, false);
			return new Integer(1);
		}
	}

	private void printUsage() {
		System.out.println("Usage arguments:");
		System.out.println("  [-data] <workspace-directory> ");
		System.out.println("  ([-chain] <chain-file>)?");
		System.out.println("");
		System.out.println("For example:");
		System.out.println("");
		System.out.println("  launch -chain myProject.chain");
	}

}
