/*
 * Copyright (c) 2005-2007 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package fr.obeo.acceleo.gen.template.scripts.imports;

import java.util.Collection;
import java.util.Comparator;
import java.util.TreeSet;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;

import fr.obeo.acceleo.ecore.factories.EFactory;
import fr.obeo.acceleo.ecore.factories.FactoryException;
import fr.obeo.acceleo.ecore.tools.ETools;
import fr.obeo.acceleo.gen.template.TemplateConstants;
import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.gen.template.eval.ENodeException;
import fr.obeo.acceleo.gen.template.eval.LaunchManager;
import fr.obeo.acceleo.gen.template.eval.log.EvalFailure;
import fr.obeo.acceleo.gen.template.expressions.TemplateCallExpression;
import fr.obeo.acceleo.gen.template.scripts.IEvalSettings;

/**
 * Metamodel element that can be used during a code generation.
 * 
 * @author www.obeo.fr
 * 
 */
public class EvalModel implements IEvalSettings {

	/**
	 * URI of the metamodel.
	 */
	protected String uri;

	/**
	 * Root element of the metamodel.
	 */
	protected EPackage metamodel;

	/**
	 * Indicates if the errors are shown.
	 */
	private boolean logErrors;

	/**
	 * Constructor.
	 * 
	 * @param uri
	 *            is the URI of the metamodel
	 */
	public EvalModel(String uri) {
		this(uri, true);
	}

	/**
	 * Constructor.
	 * 
	 * @param uri
	 *            is the URI of the metamodel
	 * @param logErrors
	 *            indicates if the errors are shown
	 */
	public EvalModel(String uri, boolean logErrors) {
		this.uri = uri;
		this.logErrors = logErrors;
		this.metamodel = ETools.uri2EPackage(uri, false);
	}

	/**
	 * @return the URI of the metamodel
	 */
	public String getUri() {
		return uri;
	}

	/**
	 * @return the root element of the metamodel
	 */
	public EPackage getMetamodel() {
		return metamodel;
	}

	/* (non-Javadoc) */
	public ENode eGet(TemplateCallExpression call, ENode node, ENode[] args, LaunchManager mode, boolean recursiveSearch) throws FactoryException, ENodeException {
		if (args.length == 0) {
			return eGetSub(call, node, args, mode);
		} else {
			return null;
		}
	}

	private ENode eGetSub(TemplateCallExpression call, ENode node, ENode[] args, LaunchManager mode) throws FactoryException, ENodeException {
		EObject eObject;
		try {
			eObject = node.getEObject();
			node = new ENode(eObject, call, mode.isSynchronize()); // for text
			// to model
		} catch (ENodeCastException e) {
			eObject = null;
		}
		ENode eval;
		if (eObject != null && EFactory.eValid(eObject, call.getLink()) && call.countArguments() == 0) {
			Object object = EFactory.eGet(eObject, call.getLink());
			eval = eGetHolder(object, node);
			EStructuralFeature feature = EFactory.eStructuralFeature(eObject, call.getLink());
			boolean containment = feature == null || !(feature instanceof EReference) || ((EReference) feature).isContainment();
			eval.setContainment(containment);
			boolean optional = feature != null && feature instanceof EReference && ((EReference) feature).getLowerBound() == 0;
			eval.setOptional(optional);
		} else {
			eval = null;
		}
		if (node.isEObject() && (eval == null || eval.isNull())) {
			if (call.getNextCall() != null && call.getNextCall().getLink().equals("naming") && call.countArguments() == 0 && call.getNextCall().countArguments() == 0) {
				try {
					// Sample : 'link.naming' is empty => try 'linkValue'
					String otherLink = call.getLink() + "Value";
					String result = EFactory.eGetAsString(eObject, otherLink);
					if (result != null) {
						ENode otherEval = new ENode(result, node);
						if (logErrors) {
							otherEval.log().addError(new EvalFailure("Evaluate " + otherLink + " on " + node.getEObject().eClass().getName() + " because " + call.getLink() + ".naming is empty"));
						}
						call.getNextCall().ignoreNextEval();
						return otherEval;
					}
				} catch (ENodeCastException e) {
					// Never catch
				} catch (FactoryException e) {
					eval = null;
				}
			}
		}
		return eval;
	}

	private ENode eGetHolder(Object object, ENode node) throws ENodeException {
		ENode result = ENode.createTry(object, node);
		if (result == null) {
			result = new ENode(ENode.EMPTY, node);
		}
		return result;
	}

	/* (non-Javadoc) */
	public Object resolveType(Object type, TemplateCallExpression call, int depth) {
		if (type != null && call.countArguments() == 0) {
			// Remark : type == GENERIC_TYPE => type != null
			if (type instanceof EClass) {
				EStructuralFeature feature = ((EClass) type).getEStructuralFeature(call.getLink());
				if (feature != null) {
					return feature.getEType();
				}
			} else if (type == GENERIC_TYPE) {
				return type;
			}
		}
		return null;
	}

	/* (non-Javadoc) */
	public Object[] getCompletionProposals(Object type, int depth) {
		if (type != null && type instanceof EClass) {
			Collection features = new TreeSet(new Comparator() {
				public int compare(Object arg0, Object arg1) {
					EStructuralFeature f0 = ((EStructuralFeature) arg0);
					EStructuralFeature f1 = ((EStructuralFeature) arg1);
					return f0.getName().compareTo(f1.getName());
				}
			});
			features.addAll(((EClass) type).getEAllStructuralFeatures());
			return features.toArray();
		} else {
			return new Object[] {};
		}
	}

	/* (non-Javadoc) */
	public boolean validateCall(TemplateCallExpression call) {
		return "".equals(call.getPrefix()) || TemplateConstants.LINK_PREFIX_METAMODEL.equals(call.getPrefix());
	}

}
