/*
 * Copyright (c) 2005-2007 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package fr.obeo.acceleo.gen.ui.editors.reflective;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.provider.EcoreEditPlugin;
import org.eclipse.emf.ecore.util.FeatureMap;
import org.eclipse.emf.edit.provider.ComposedImage;
import org.eclipse.emf.edit.provider.ReflectiveItemProvider;

import fr.obeo.acceleo.ecore.factories.EFactory;
import fr.obeo.acceleo.ecore.factories.FactoryException;
import fr.obeo.acceleo.ecore.tools.ELoaderUtils;
import fr.obeo.acceleo.ecore.tools.ETools;
import fr.obeo.acceleo.gen.template.TemplateConstants;
import fr.obeo.acceleo.gen.ui.AcceleoEcoreGenUiPlugin;
import fr.obeo.acceleo.tools.plugins.AcceleoModuleProvider;
import fr.obeo.acceleo.tools.resources.Resources;

/**
 * Specific item provider for the reflective editor.
 * 
 * @author www.obeo.fr
 * 
 */
public class AcceleoReflectiveItemProvider extends ReflectiveItemProvider {

	/**
	 * Constructor.
	 * 
	 * @param adapterFactory
	 *            is the adapter factory
	 */
	public AcceleoReflectiveItemProvider(AcceleoReflectiveItemProviderAdapterFactory adapterFactory) {
		super(adapterFactory);
	}

	/**
	 * Get the reflective editor.
	 * 
	 * @return reflective editor
	 */
	protected AcceleoReflectiveEditor getReflectiveEditor() {
		return ((AcceleoReflectiveItemProviderAdapterFactory) adapterFactory).getReflectiveEditor();
	}

	/**
	 * Get the input file for the corresponding reflective editor.
	 * 
	 * @return the input file
	 */
	protected IFile getFile() {
		return getReflectiveEditor().getFile();
	}

	/**
	 * Try to get an eClass icon in /icons folder. The hierarchy of icons must
	 * correspond to the hierarchy in the metamodel. If the icon of an object
	 * doesn't exist, the icon DEFAULT.gif of the current directory is used.
	 * 
	 * @param eClass
	 * @return icon URI
	 */
	protected URI tryToFindImageURI(EClass eClass) {
		// Sample : "java.resources.Folder"
		String name = ETools.getEClassifierPath(eClass);
		// Search in folder /icons
		IFile file = getFile();
		if (file != null) {
			IPath path = new Path("/icons/" + name.replaceAll("\\.", "/") + ".gif");
			IFile metamodel = ELoaderUtils.xmi2ecore(file);
			if (metamodel != null) {
				IFile icon = metamodel.getParent().getFile(path);
				if (icon.exists()) {
					URI uri = Resources.createPlatformResourceURI(icon.getFullPath().toString());
					return uri;
				} else {
					// Possibly DEFAULT.gif
					path = path.removeLastSegments(1).append(new Path("DEFAULT.gif"));
					icon = metamodel.getParent().getFile(path);
					if (icon.exists()) {
						URI uri = Resources.createPlatformResourceURI(icon.getFullPath().toString());
						return uri;
					}
				}
			}
			path = new Path("model").append(path);
			URI uri = getIconsInBundles(file.getProject(), path);
			if (uri == null) {
				// Possibly DEFAULT.gif
				path = path.removeLastSegments(1).append(new Path("DEFAULT.gif"));
				uri = getIconsInBundles(file.getProject(), path);
			}
			return uri;
		} else {
			return null;
		}
	}

	private URI getIconsInBundles(IProject project, IPath path) {
		if (iconsBundleId != null) {
			File file = AcceleoModuleProvider.getDefault().getFile(iconsBundleId, path);
			if (file != null) {
				return URI.createFileURI(file.getAbsolutePath());
			}
		} else {
			if (project.exists()) {
				String[] requiredPluginIDs = Resources.getRequiredPluginIDs(project);
				for (int i = 0; i < requiredPluginIDs.length; i++) {
					File file = AcceleoModuleProvider.getDefault().getFile(requiredPluginIDs[i], path);
					if (file != null) {
						iconsBundleId = requiredPluginIDs[i];
						return URI.createFileURI(file.getAbsolutePath());
					}
				}
			}
		}
		return null;
	}

	private String iconsBundleId = null;

	/* (non-Javadoc) */
	public Object getImage(Object object) {
		EObject eObject = (EObject) object;
		Object image = getEClassImage(eObject);
		// Decorate icon when object hasError or when object is generated
		if (getReflectiveEditor().getActivePreviewEditor() != null) {
			boolean hasError = getReflectiveEditor().getActivePreviewEditor().getSettings().getScript().hasError(eObject);
			boolean isGenerated = getReflectiveEditor().getActivePreviewEditor().getSettings().getScript().isGenerated(eObject);
			return decorateImage(image, hasError, isGenerated);
		} else {
			return decorateImage(image, false, false);
		}
	}

	private Object getEClassImage(EObject eObject) {
		EClass eClass = eObject.eClass();
		// Try to get eClass icon
		String eClassName = ETools.getEClassifierShortPath(eClass);
		Object image = eClassName2Image.get(eClassName);
		if (image == null && !eClassName2Image.containsKey(eClassName)) {
			image = tryToFindImageURI(eClass);
			if (image == null) {
				image = getSpecificImage(eObject);
				if (image == null) {
					image = URI.createURI(getResourceLocator().getImage("full/obj16/Item").toString() + "#" + eClass.getName());
				}
			}
			eClassName2Image.put(eClassName, image);
		}
		return image;
	}

	private Map eClassName2Image = new HashMap();

	/**
	 * Gets the specific image for the given EObject
	 * 
	 * @param object
	 *            is an object of the model
	 * @return the image
	 */
	protected Object getSpecificImage(EObject object) {
		if (object == null) {
			return null;
			// Images for uml2
		} else if (object.getClass().getName().startsWith("org.eclipse.uml2")) {
			String className = object.getClass().getName();
			if (className.endsWith("Impl")) {
				className = className.substring(0, className.length() - 4);
			}
			int iDot = className.lastIndexOf(".");
			if (iDot > -1) {
				className = className.substring(iDot + 1);
			}
			if (className.equals("Package")) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EPackage.gif")).toString());
			} else if (className.equals("Association")) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EReference.gif")).toString());
			} else if (className.equals("AssociationClass")) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EReference.gif")).toString());
			} else if (className.equals("Extension")) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EReference.gif")).toString());
			} else if (className.equals("Property")) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EAttribute.gif")).toString());
			} else if (className.equals("ExtensionEnd")) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EAttribute.gif")).toString());
			} else if (className.equals("Package")) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EPackage.gif")).toString());
			} else if (className.equals("Model")) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EPackage.gif")).toString());
			} else if (className.equals("Operation")) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EOperation.gif")).toString());
			} else if (className.equals("Parameter")) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EParameter.gif")).toString());
			} else if (className.equals("Class")) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EClass.gif")).toString());
			} else if (className.equals("Interface")) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EClass.gif")).toString());
			} else if (className.equals("DataType")) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EClass.gif")).toString());
			} else if (className.equals("Enumeration")) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EClass.gif")).toString());
			} else if (className.equals("PrimitiveType")) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EClass.gif")).toString());
			} else {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EObject.gif")).toString());
			}
			// Images for uml14
		} else if (object instanceof fr.obeo.acceleo.uml14.foundation.core.Element) {
			if (object instanceof fr.obeo.acceleo.uml14.model_management.Package) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EPackage.gif")).toString());
			} else if (object instanceof fr.obeo.acceleo.uml14.foundation.core.Classifier) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EClass.gif")).toString());
			} else if (object instanceof fr.obeo.acceleo.uml14.foundation.core.Attribute) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EAttribute.gif")).toString());
			} else if (object instanceof fr.obeo.acceleo.uml14.foundation.core.AssociationEnd) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EReference.gif")).toString());
			} else if (object instanceof fr.obeo.acceleo.uml14.foundation.core.Association) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EObject.gif")).toString());
			} else if (object instanceof fr.obeo.acceleo.uml14.foundation.core.Operation) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EOperation.gif")).toString());
			} else if (object instanceof fr.obeo.acceleo.uml14.foundation.core.Method) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EOperation.gif")).toString());
			} else if (object instanceof fr.obeo.acceleo.uml14.foundation.core.Parameter) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EParameter.gif")).toString());
			} else {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EObject.gif")).toString());
			}
			// Images for uml13
		} else if (object instanceof fr.obeo.acceleo.uml13.foundation.core.Element) {
			if (object instanceof fr.obeo.acceleo.uml13.model_management.Package) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EPackage.gif")).toString());
			} else if (object instanceof fr.obeo.acceleo.uml13.foundation.core.Classifier) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EClass.gif")).toString());
			} else if (object instanceof fr.obeo.acceleo.uml13.foundation.core.Attribute) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EAttribute.gif")).toString());
			} else if (object instanceof fr.obeo.acceleo.uml13.foundation.core.AssociationEnd) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EReference.gif")).toString());
			} else if (object instanceof fr.obeo.acceleo.uml13.foundation.core.Association) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EObject.gif")).toString());
			} else if (object instanceof fr.obeo.acceleo.uml13.foundation.core.Operation) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EOperation.gif")).toString());
			} else if (object instanceof fr.obeo.acceleo.uml13.foundation.core.Method) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EOperation.gif")).toString());
			} else if (object instanceof fr.obeo.acceleo.uml13.foundation.core.Parameter) {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EParameter.gif")).toString());
			} else {
				return URI.createURI(EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/EObject.gif")).toString());
			}
		} else {
			URL url = EcoreEditPlugin.getPlugin().find(new Path("/icons/full/obj16/" + object.eClass().getName() + ".gif"));
			if (url != null) {
				return URI.createURI(url.toString());
			} else {
				return null;
			}
		}
	}

	/**
	 * Decorate image when object hasError or when object is generated
	 * 
	 * @param image
	 * @param hasError
	 *            indicate if 'error' decorator is necessary
	 * @param isGenerated
	 *            indicate if 'generate' decorator is necessary
	 * @return the decorated image
	 */
	protected Object decorateImage(Object image, final boolean hasError, final boolean isGenerated) {
		if (hasError || isGenerated) {
			Collection images = new ArrayList();
			if (image == null) {
				image = AcceleoEcoreGenUiPlugin.getDefault().getImage("/icons/proposals/predefined.gif");
			}
			images.add(image);
			// hasError
			if (hasError) {
				images.add(AcceleoEcoreGenUiPlugin.getDefault().getImage("/icons/reflective/error.gif"));
			}
			// isGenerated
			if (isGenerated) {
				images.add(AcceleoEcoreGenUiPlugin.getDefault().getImage("/icons/reflective/generate.gif"));
			}
			if (images.size() > 1) {
				return new ComposedImage(images) {
					public List getDrawPoints(Size size) {
						List result = super.getDrawPoints(size);
						if (hasError && isGenerated) {
							((Point) result.get(0)).y = 0;
							((Point) result.get(1)).y = 7;
							((Point) result.get(2)).x = 3;
							((Point) result.get(2)).y = 4;
						} else if (hasError) {
							((Point) result.get(0)).y = 0;
							((Point) result.get(1)).y = 7;
						} else if (isGenerated) {
							((Point) result.get(0)).y = 0;
							((Point) result.get(1)).x = 3;
							((Point) result.get(1)).y = 4;
						}
						return result;
					}
				};
			}
		}
		return image;
	}

	/* (non-Javadoc) */
	public String getText(Object object) {
		return getText(object, getReflectiveEditor().showType);
	}

	private String getText(Object object, boolean showType) {
		EObject eObject = (EObject) object;
		String displayType;
		if (showType) {
			displayType = " [" + ((eObject.eContainingFeature() != null) ? eObject.eContainingFeature().getName() + ":" : "") + eObject.eClass().getName() + "]"
					+ ((eObject.eContainer() != null) ? " <<" + getText(eObject.eContainer(), false) + ">>" : "");
		} else {
			displayType = "";
		}
		try {
			if (EFactory.eInstanceOf(eObject, "resources.FileLog")) {
				// FileLog => show the number of elements
				List logs = EFactory.eGetAsList(eObject, "logs");
				if (logs != null && logs.size() > 0)
					return "Errors:" + logs.size() + displayType;
			} else if (EFactory.eInstanceOf(eObject, "resources.Log")) {
				// Log => show the message
				String message = EFactory.eGetAsString(eObject, "message");
				if (message != null)
					return message + displayType;
			} else {
				// others => show naming attribute
				String naming = EFactory.eGetAsString(eObject, "naming");
				if (naming != null && naming.length() > 0)
					return naming + displayType;
			}
		} catch (FactoryException e) {
		}
		if (EFactory.eValid(eObject, "naming")) {
			return TemplateConstants.FEATURE_BEGIN + "naming" + TemplateConstants.FEATURE_END + displayType;
		} else {
			return super.getText(object) + displayType;
		}
	}

	/* (non-Javadoc) */
	protected EStructuralFeature getLabelFeature(EClass eClass) {
		EAttribute result = null;
		for (Iterator i = eClass.getEAllAttributes().iterator(); i.hasNext();) {
			EAttribute eAttribute = (EAttribute) i.next();
			if (!eAttribute.isMany() && eAttribute.getEType().getInstanceClass() != FeatureMap.Entry.class) {
				if ("name".equalsIgnoreCase(eAttribute.getName())) {
					result = eAttribute;
					break;
				} else if (result == null) {
					result = eAttribute;
				} else if (eAttribute.getEAttributeType().getInstanceClass() == String.class && result.getEAttributeType().getInstanceClass() != String.class) {
					result = eAttribute;
				}
			}
		}
		return result;
	}

	/* (non-Javadoc) */
	public Object getCreateChildImage(Object owner, Object feature, Object child, Collection selection) {
		if (feature instanceof EReference && child instanceof EObject) {
			EReference reference = (EReference) feature;
			EClass parentClass = reference.getEContainingClass();
			EClass childClass = ((EObject) child).eClass();
			// Try to get childClass icon
			String childClassName = ETools.getEClassifierShortPath(childClass);
			Object image = eClassName2Image.get(childClassName);
			if (image == null && !eClassName2Image.containsKey(childClassName)) {
				image = tryToFindImageURI(childClass);
				if (image == null) {
					image = getSpecificImage((EObject) child);
				}
				eClassName2Image.put(childClassName, image);
			}
			if (image != null)
				return image;
			else
				return URI.createURI(getResourceLocator().getImage("full/ctool16/CreateChild").toString() + "#" + parentClass.getName() + "/" + childClass.getName());
		}
		return getResourceLocator().getImage("full/ctool16/CreateChild");
	}

	/* (non-Javadoc) */
	protected String getTypeText(Object object) {
		return object instanceof EObject ? ((EObject) object).eClass().getName() : getString("_UI_Unknown_type");
	}

	/* (non-Javadoc) */
	protected String getFeatureText(Object feature) {
		return feature instanceof EStructuralFeature ? ((EStructuralFeature) feature).getName() : getResourceLocator().getString("_UI_Unknown_feature");
	}

	/* (non-Javadoc) */
	public void fireNotifyChanged(Notification notification) {
		super.fireNotifyChanged(notification);
		int type = notification.getEventType();
		if (type != Notification.REMOVING_ADAPTER && type != Notification.RESOLVE) {
			if (getReflectiveEditor().getActivePreviewEditor() != null) {
				getReflectiveEditor().getActivePreviewEditor().refresh();
			}
		}
	}

}
