/*
 * Copyright (c) 2005-2007 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package fr.obeo.acceleo.gen.ui.wizards;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;

import fr.obeo.acceleo.gen.template.Template;
import fr.obeo.acceleo.gen.template.TemplateSyntaxException;
import fr.obeo.acceleo.gen.template.scripts.ScriptDescriptor;
import fr.obeo.acceleo.gen.template.scripts.SpecificScript;
import fr.obeo.acceleo.gen.template.scripts.imports.EvalModel;
import fr.obeo.acceleo.gen.ui.AcceleoEcoreGenUiPlugin;

/**
 * The role of this wizard is to create an empty template.
 * 
 * @author www.obeo.fr
 */
public class AcceleoNewEmptyTemplateWizard extends Wizard implements INewWizard {

	/**
	 * The page to select a metamodel.
	 */
	protected AcceleoSelectMetamodelWizardPage pageSelectMetamodel;

	/**
	 * The page to create a new file.
	 */
	protected AcceleoNewFileWizardPage pageNewFile;

	/**
	 * The current selection.
	 */
	protected ISelection selection;

	/**
	 * Constructor.
	 */
	public AcceleoNewEmptyTemplateWizard() {
		super();
		setNeedsProgressMonitor(true);
	}

	/* (non-Javadoc) */
	public void addPages() {
		addPage(pageSelectMetamodel = new AcceleoSelectMetamodelWizardPage("Select a metamodel (1/2)", new String[] { "ecore" }, true));
		addPage(pageNewFile = new AcceleoNewFileWizardPage("Create a new template (2/2)", selection, SpecificScript.GENERATORS_EXTENSION));
	}

	/* (non-Javadoc) */
	public boolean performFinish() {
		final String metamodelURI = pageSelectMetamodel.getMetamodelURI();
		final String metamodelType = pageSelectMetamodel.getMetamodelType();
		final String containerName = pageNewFile.getContainerName();
		final String fileName = pageNewFile.getFileName();
		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException {
				try {
					doFinish(metamodelURI, metamodelType, containerName, fileName, monitor);
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};
		try {
			getContainer().run(true, false, op);
		} catch (InterruptedException e) {
			return false;
		} catch (InvocationTargetException e) {
			AcceleoEcoreGenUiPlugin.getDefault().log(e.getTargetException(), true);
			return false;
		}
		return true;
	}

	private void doFinish(String metamodelURI, String metamodelType, String containerName, String fileName, IProgressMonitor monitor) throws CoreException {
		monitor.beginTask("Creating " + fileName, 2);
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IResource resource = root.findMember(new Path(containerName));
		if (resource == null || !resource.exists() || !(resource instanceof IContainer)) {
			AcceleoEcoreGenUiPlugin.getDefault().log("Container \"" + containerName + "\" does not exist.", true);
		} else {
			IContainer container = (IContainer) resource;
			IPath filePath = new Path(fileName);
			final IFile file = container.getFile(filePath);
			try {
				InputStream stream = openContentStream(metamodelURI, metamodelType, filePath.removeFileExtension().lastSegment());
				if (file.exists()) {
					file.setContents(stream, true, true, monitor);
				} else {
					file.create(stream, true, monitor);
				}
				stream.close();
			} catch (TemplateSyntaxException e) {
				AcceleoEcoreGenUiPlugin.getDefault().log(e, true);
			} catch (IOException e) {
				AcceleoEcoreGenUiPlugin.getDefault().log(e, true);
			}
			monitor.worked(1);
			monitor.setTaskName("Opening file for editing...");
			getShell().getDisplay().asyncExec(new Runnable() {
				public void run() {
					IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
					try {
						IDE.openEditor(page, file, true);
					} catch (PartInitException e) {
					}
				}
			});
			monitor.worked(1);
		}
	}

	/**
	 * Creates the content of the template.
	 * 
	 * @param metamodelURI
	 *            is the metamodel URI
	 * @param metamodelType
	 *            is the type of the root object
	 * @param scriptName
	 *            is the name of the script
	 * @return the content
	 * @throws TemplateSyntaxException
	 */
	private InputStream openContentStream(String metamodelURI, String metamodelType, String scriptName) throws TemplateSyntaxException {
		// Templates of the script
		SpecificScript script = createSpecificScript(metamodelURI, metamodelType, scriptName);
		script.addImport(new EvalModel(metamodelURI));
		String content = script.toString();
		return new ByteArrayInputStream(content.getBytes());
	}

	/**
	 * Creates the script to serialize.
	 * 
	 * @param metamodelURI
	 *            is the metamodel URI
	 * @param metamodelType
	 *            is the type of the root object
	 * @param scriptName
	 *            is the name of the script
	 * @return the default script
	 * @throws TemplateSyntaxException
	 */
	protected SpecificScript createSpecificScript(String metamodelURI, String metamodelType, String scriptName) throws TemplateSyntaxException {
		SpecificScript script = new SpecificScript();
		Template textTemplate = script.createTextTemplate(new ScriptDescriptor(metamodelType, scriptName));
		script.createFileTemplate(metamodelType, "<%name%>.txt", textTemplate);
		return script;
	}

	/* (non-Javadoc) */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
	}

}
